package comtestplug.popup.actions;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.action.Action;
//import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.jface.dialogs.MessageDialog;

import comtest.ComTest;

/**
 * <p>Test generator action for ComTest to by used from Eclipse.</p>
 * 
 * <p>I needed to implement also IWorkbenchWindowActionDelegate
 * and extend Action to get key shortcuts to work.
 * Now the problem is thata there is two plugin instances.
 * One for PopUp menus that changes the selection.
 * And one for "windows" that does not carry out the file name.
 * So the filename from PopUp is saved to
 * <pre>
 *   private static Collection<IFile> staticfiles = new ArrayList<IFile>();
 * </pre>  
 * static array so it can be found from the other instance.
 * Actually this works quite nice.  There is allways the last from
 * PopUp runned file name that can be runned anywhere in Eclipse by key shortcut.</p> 
 * 
 * @author vesal, panu, ji
 *
 */
public class GenerateTest extends Action implements IObjectActionDelegate,IWorkbenchWindowActionDelegate {
  private Collection<IFile> files;
  private IWorkbenchWindow window = null;

  /**
   * Constructor for Action1.
   */
  public GenerateTest() {
    super();
    files = new ArrayList<IFile>();
  }

  /**
   * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
   */
  @Override
  public void setActivePart(IAction action, IWorkbenchPart targetPart) {}

  /**
   * Method to start plugin
   */
  @Override
  public void run() {
      run((IAction)null);
  }
  
  /**
   * @see IActionDelegate#run(IAction)
   */
  @Override
  public void run(IAction action) {
      
    if ( window != null ) {
        selectionChanged(action, window.getSelectionService().getSelection());
    }
      
    final IWorkspace workspace = ResourcesPlugin.getWorkspace();

    try{
      
      // Run ComTest in its own task.
      final ComtestRun cr = new ComtestRun(files);
      if ( !cr.checkFiles(window) ) return;
      workspace.run(cr,workspace.getRoot(), 0, null);

      // Build are affected projects.
      // But because jdt.core could not be loaded
      // we wait the completion before doing anything.
      for (final IProject project : cr.getAffectedProjects()){
          buildDone(cr.getOutFiles());
          /*  // for some reason in Eclipse 19.12 the OnceDonner.done is not fired??? 
        project.build(IncrementalProjectBuilder.INCREMENTAL_BUILD, new OnceDonner(){

          @Override
          protected void doTheDone() throws CoreException {
            // Stubid HACK. Building projects prompts
            // file savings so run ComTest AGAIN to use saved files.
            workspace.run(cr,workspace.getRoot(), 0, new OnceDonner(){
              @Override
              protected void doTheDone() throws CoreException {
                // And because of new code generated we need to build once more.
                project.build(IncrementalProjectBuilder.INCREMENTAL_BUILD, new OnceDonner(){
                  @Override
                  protected void doTheDone() throws CoreException {
                    // Call build done method for after build parties.
                    buildDone(cr.getOutFiles());
                  }              
                });
              }
            });
          }
        });
        */
      }



    } catch (CoreException e){
      e.printStackTrace(); 
    }
  }

  /**
   * @see IActionDelegate#selectionChanged(IAction, ISelection)
   */
  @Override
  public void selectionChanged(IAction action, ISelection selection) {
    try {
      if ( !( selection instanceof IStructuredSelection ) ) return;  
      IStructuredSelection sel = (IStructuredSelection) selection;
      doSelection(sel);
    } catch (ClassCastException cce) {
      doSelection(selection);
    }
  }

  private void doSelection(IStructuredSelection selection) {
    boolean first = true;  
    try {
      for (Iterator<IFile> items = selection.iterator(); items.hasNext();) {
        IFile file = items.next();
        if ( first) files.clear(); first = false;
        files.add(file);
      }
    } catch (ClassCastException e) {
      // System.err.println("Items:" + selection.toString());
      // e.printStackTrace();
    }
  }

  /**
   * Method exists for overriding purposes.
   * With this generated files can be further processed.
   * For example run with JUnit.
   * This implementation does nothing.
   * @param testFiles
   */
  protected void buildDone(Collection<IFile> testFiles) {}

  @SuppressWarnings("static-method")
  private void doSelection(ISelection sel) {
     System.err.println("something different" + sel.toString());
  }



  static class ComtestRun implements IWorkspaceRunnable{
    private static Collection<IFile> staticfiles = new ArrayList<IFile>();
    Collection<IFile> myInFiles = new Vector<IFile>();
    Set<IProject> myAffectedProjects =new HashSet<IProject>();
    Set<IFile> myOutFiles = new HashSet<IFile>();
    
    /**
     * Run to plugin using files
     * @param files to make tests
     */
    public ComtestRun(Collection<IFile> files){
      if ( files.size() > 0 ) {
        this.myInFiles.addAll(files);
      }  
      if ( myInFiles.size() == 0 ) this.myInFiles.addAll(staticfiles);
 
    }
    
    /**
     * Check if there is no saved files.  When run as action,
     * we must use the saved files.
     * @param window where to show warning
     * @return how it succeeds
     */
    public boolean checkFiles(IWorkbenchWindow window) {
        if ( myInFiles.size() > 0 ) return true;
        if ( window == null ) return true;
        MessageDialog.openInformation(
                    window.getShell(),
                    "ComTest warning",
                    "Run first one time from PopUp menu");
        return false;
    }
    
    /**
     * Run the plugin
     * @param monitor 
     * @throws CoreException
     */
    @Override
    @SuppressWarnings("static-access")
    public void run(IProgressMonitor monitor) throws CoreException {
      boolean goodfiles = false;  
      for(IFile file : myInFiles){
        final IProject project = file.getProject();
        final IPath  inPath = file.getLocation();
        final String inname = inPath.toString();
        String lowername = inname.toLowerCase(); 
        if ( lowername.endsWith("test.java") && ! lowername.endsWith("comtest.java") ) {
            if ( myInFiles.size() <= 1)
              MessageDialog.openInformation(
                    null,
                    "ComTest warning",
                    "Do not run for names ending with test.java: \n" + inname);
           continue; // do not run for test-ending names   
        }

        /*IJavaElement je = (IJavaElement) file.getAdapter(IJavaElement.class);
        if (je instanceof ICompilationUnit){
          ICompilationUnit cu = (ICompilationUnit) je;


          if (cu.isWorkingCopy()){
            cu.commitWorkingCopy(false, null);
          } else {
            cu.save(null, false);
          }*/
          
          String outname = ComTest.processFile(inname);
          if (outname != null) {
            IPath projectLocation = project.getLocation();
            IPath outPath = new Path(outname);
            int count = outPath.matchingFirstSegments(projectLocation);
            if (count == projectLocation.segmentCount()){
              // Result is under project.
              IFile outFile = project.getFile(outPath.removeFirstSegments(count));
              myAffectedProjects.add(outFile.getProject());
              outFile.getParent().refreshLocal(IResource.DEPTH_INFINITE, null);
              myOutFiles.add(outFile);
              goodfiles = true;
            }          
          } else{
            //TODO Error logging and stuff. 
            System.err.println("ERROR> " + inname);
          }
        //}        
      }
      if ( goodfiles ) {
        staticfiles.clear();
        this.staticfiles.addAll(myInFiles); // save found files
      }  

    }  
    
    /**
     * Return projects that was affected
     * @return affected projects
     */
    public Collection<IProject> getAffectedProjects(){
      return myAffectedProjects;
    }

    /**
     * Get files that was output
     * @return files that was output
     */
    public Collection<IFile> getOutFiles(){
      return myOutFiles;
    }
  }

  static abstract class OnceDonner extends NullProgressMonitor {
    
    /** 
     * Method that must be implemented 
     * @throws CoreException when error 
     * 
     */  
    protected abstract  void doTheDone() throws CoreException;

    boolean alreadyDone =false;
    
    /**
     * Method to call when done
     */
    @Override
    public final void done() {
      if (alreadyDone){return;}
      alreadyDone = true;
      try{
        doTheDone();
      } catch(CoreException e){
        e.printStackTrace();
      }
    }

    /**
     * Return if it is done
     * @return if it is done
     */
    public boolean isDone(){
      return alreadyDone;
    }
    
    
    @Override
    public void worked(int work) {
       // System.err.println("Work: " + work);
    }
    
    
  }

    /**
     *  Method to clean things 
     */
    @Override
    public void dispose() {
        window = null;
    }

    /**
     * Method to init things
     * @param window to use as source
     */
    @Override
    public void init(IWorkbenchWindow window) {
        this.window = window;
        return;

    }
}
