package comtestplug.ui.text.java;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.IProgressMonitor;

import org.eclipse.jdt.ui.text.java.ContentAssistInvocationContext;
import org.eclipse.jdt.ui.text.java.IJavaCompletionProposalComputer;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.contentassist.CompletionProposal;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.IContextInformation;

import comtest.StringUtilities;

/**
 * Class for plugin text to fill
 * @author panu
 *
 */
public class ComtestProposalComputer implements IJavaCompletionProposalComputer {
  final static String[] proposals = {"#STATICIMPORT","#NOIMPORT","#DIRECTORY=","#PACKAGE=",
    "#DYNAMICIMPORT","#CLASSIMPORT","#PACKAGEIMPORT",  
    "#COLUMNSEPARATOR","#COLUMNCHAR","#THROWS","#import"};
  List<String> baseProposals = Arrays.asList(proposals);

  /**
   * @param context
   * @param monitor
   * @return list of proposals
   */
  @Override
public List<ICompletionProposal> computeCompletionProposals(
      ContentAssistInvocationContext context, IProgressMonitor monitor) {
    List<ICompletionProposal> ret = new Vector<ICompletionProposal>();
    try {      
      int offs = context.getInvocationOffset();
      String buffer = context.getDocument().get(0, offs);
      for(String proposal : proposals){
        int index = StringUtilities.countEndIndex(buffer, proposal);
        if (index == 0) continue;
        String replace = proposal.substring(index);
        ICompletionProposal ci = new CompletionProposal(
            replace, offs, 0, replace.length(),
            null,proposal,null,null); 
        ret.add(ci);
      }
      int index = StringUtilities.countEndIndex(buffer, "comtest");
      if (index == 0) return ret;
      String indent = StringUtilities.getLineIndent(buffer,offs);
      String comtestBlock = "@example\n"+indent+"* <pre name=\"test\">\n"+indent+"* \n"+indent+"* </pre>";
      int p = comtestBlock.indexOf("* \n");
      
      ICompletionProposal ci = new CompletionProposal(
        comtestBlock,
        offs-index, index, p+2,
        null,"ComTest example",null,null); 
       ret.add(ci);
    } catch (BadLocationException e) {
      e.printStackTrace();
    }
    
    
    return ret;
  }

  /**
   * @param context
   * @param monitor
   * @return list of proposals
   */
  @Override
  public List<IContextInformation> computeContextInformation(
      ContentAssistInvocationContext context, IProgressMonitor monitor) {
    // TODO Auto-generated method stub
    return new ArrayList<IContextInformation>();
  }

  /**
   * @return error message from class
   */ 
  @Override
  public String getErrorMessage() {
    return "Error from ComTest";
  }

  /**
   * 
   */
  @Override
  public void sessionEnded() {
  }

  /**
   * 
   */
  @Override
  public void sessionStarted() {
  }

}
